import { world, system, EquipmentSlot, ItemStack, Player, GameMode, MolangVariableMap, BlockPermutation} from '@minecraft/server';
Player.prototype.getGameMode = function() {
    return Object.values(GameMode).find((gameMode) => world.getPlayers({ name: this.name, gameMode }).length > 0)
};
const molang = new MolangVariableMap();
function destroyPlane(target) {
    target.getProperty('server:dispenser_upgrade') ? target.dimension.spawnItem(new ItemStack("minecraft:dispenser", 1), target.location) : undefined;
    target.getProperty('server:sensor_upgrade') ? target.dimension.spawnItem(new ItemStack("minecraft:daylight_detector", 1), target.location) : undefined;
    if (target.getProperty('server:fuel') > 20) {target.dimension.spawnItem(new ItemStack("minecraft:redstone", Math.floor(target.getProperty('server:fuel')/22)), target.location)};
    target.kill();
}
let regen;
world.afterEvents.entityHurt.subscribe(eventData => {
    if (regen) system.clearRun(regen);
    let target = eventData.hurtEntity;
    target.playAnimation('animation.plane.shake');
    if (target.typeId === "server:plane" && target.getComponent("health").currentValue <= 16 && target.getComponent("health").currentValue > 1) {
        destroyPlane(target);
    }
    regen = system.runTimeout(() => {
        try {target.addEffect("instant_health", 5, {amplifier:1, showParticles:false})} catch (error) {}
    }, 30);
});
let timer = 0;
function getBarDisplay(currentValue, maxValue, barLength) {
    const segments = Math.ceil(currentValue / maxValue * barLength);
    let barDisplay;
    if (currentValue-5 > maxValue * 0.5) {
        barDisplay = "§a";
    } else if (currentValue-5 > maxValue * 0.25) {
        barDisplay = "§6";
    } else {barDisplay = "§c";}
        for (let i = 0; i < segments; i++) {
            barDisplay += "█";
        };
        barDisplay += "§8";
        for (let i = segments; i < barLength; i++) {
            barDisplay += "▒";
        };
    return barDisplay;
};
let riding;
function fly() {
    for (const player of world.getAllPlayers()) {
        if (player.cameraToggle === undefined) player.cameraToggle = true;
        riding = player.getComponent( "minecraft:riding" )?.entityRidingOn;
        if (riding?.typeId === "server:plane") {
            player.accerlationRecord = Math.abs(riding.getVelocity().y.toFixed(2));
            if (riding.isOnGround || riding.isInWater) {player.runCommandAsync(`stopsound @s server.plane.fly`)};
            if (player.hasTag("smoke")) { 
                player.dimension.spawnParticle("server:smoke", {x:player.location.x, y:player.location.y-2, z:player.location.z}, molang)
            } else if (player.hasTag("dust")) {
                player.dimension.spawnParticle("server:glow", player.location)
            } else if (player.hasTag("sparkling")) {
                player.dimension.spawnParticle("server:sparkling", player.location)
            }
            player.groundCheck = riding.dimension.getBlockFromRay( riding.location , { x: 0, y: -1, z: 0 }, {includeLiquidBlocks: true, maxDistance: 3} )?.block?.typeId;
            if (!player.groundCheck && !riding.isInWater) {
                riding.addTag("activated");
                player.runCommandAsync(`ride @s start_riding @e[type=server:plane, r=10] teleport_rider`);
            }
            if (player.accerlationRecord > 3.5 && riding.hasTag("activated") && player.groundCheck && player.groundCheck !== "minecraft:water") {
                riding.kill();
                player.dimension.createExplosion(player.location, 1.5*player.accerlationRecord, { allowUnderwater: false, breaksBlocks: true, causesFire: true });
                player.accerlationRecord = 0;
            };
            player.averageVeloctiy = 0.5*(Math.abs(riding.getVelocity().x) + Math.abs(riding.getVelocity().z));
            if (player.cameraToggle) {player?.runCommandAsync(`camera @s set minecraft:free ease 0.5 linear pos ^ ^${5 + player.averageVeloctiy*35} ^-10 rot ~${5 + Math.min(player.averageVeloctiy*80, 80)} ~`)} else {player?.runCommandAsync(`camera @s clear`);}
            if (player.averageVeloctiy > 0.2 && !player.groundCheck && !riding.isInWater && (world.getTimeOfDay() < 500 || world.getTimeOfDay() > 12000 || riding.getProperty("server:sensor_upgrade") === false)) {
                timer++;
                player.onScreenDisplay.setActionBar(`§lFUEL:§r ${getBarDisplay(riding.getProperty("server:fuel"), 180, 9)} `);
            } else if (riding.getProperty("server:sensor_upgrade") && world.getTimeOfDay() > 500 && world.getTimeOfDay() < 12000) {
                player.onScreenDisplay.setActionBar(`§lFUEL:§r §o§aSOLAR POWERED `);
            } else if (riding.getProperty("server:fuel") === 0 ) {
                player.onScreenDisplay.setActionBar(`§lFUEL:§r §cOUT OF FUEL `);
                if (!riding.isInWater && !riding.isOnGround) riding.applyImpulse({x:0, y:-0.1, z:0});
            };
            if (timer > 20) {
                if (riding.getProperty("server:fuel") > 0) {
                    riding.setProperty("server:fuel", riding.getProperty("server:fuel")-1);
                };
                timer = 0;
            }
            if (riding.getProperty('server:fuel') > 0 || (riding.getProperty("server:sensor_upgrade") && world.getTimeOfDay() > 500 && world.getTimeOfDay() < 12000)) {
                if (riding?.typeId == "server:plane" && player.averageVeloctiy > 0.2 && riding.location.y < 230) {
                    riding.applyImpulse({x: player.getViewDirection().x * 0.1, y: (player.getViewDirection().y - 0.1) * 0.1 + 0.05, z: player.getViewDirection().z * 0.1});
                    riding.setProperty("server:tilt", player.getViewDirection().y*20);
                } else if (riding?.typeId == "server:plane" && player.getViewDirection().y > 0.65 && !player.groundCheck && riding.location.y < 380) {
                    riding.setProperty("server:tilt", player.getViewDirection().y*80);
                    riding.applyImpulse({x: player.getViewDirection().x * 0.5, y: player.getViewDirection().y * 0.2, z: player.getViewDirection().z * 0.5});
                } else if (riding?.typeId == "server:plane" && player.getViewDirection().y < -0.65 && !player.groundCheck && riding.location.y < 380) {
                    riding.setProperty("server:tilt", player.getViewDirection().y*80);
                    riding.applyImpulse({x: player.getViewDirection().x * 0.5, y: player.getViewDirection().y * 0.01, z: player.getViewDirection().z * 0.5});
                } else if (riding?.typeId == "server:plane" && player.averageVeloctiy > 0.2 && riding.location.y >= 230 && Math.abs(player.getViewDirection().y) < 0.65) {
                    riding.setProperty("server:tilt", player.getViewDirection().y*40);
                    riding.applyImpulse({x: player.getViewDirection().x * 0.1, y: 0.01, z: player.getViewDirection().z * 0.1});
                }
            }
        } else {
            player.runCommandAsync(`stopsound @s server.plane.fly`);
            player?.runCommandAsync(`camera @s clear`);
            player.dimension.getEntities({ type: "server:plane", maxDistance: 10, location: player?.location })[0]?.removeTag("activated");
        };
    };
};
function usedReduce(targetUseItem) {
    if (targetUseItem.getGameMode() !== "survival") {return;}
    let currentItem = targetUseItem.getComponent("minecraft:equippable").getEquipment(EquipmentSlot.Mainhand);
    let afterItem;
    if (currentItem.amount > 1) {
        afterItem = new ItemStack(currentItem.typeId, currentItem.amount - 1);
    } else {
        afterItem = undefined;
    }
    targetUseItem.getComponent("minecraft:equippable").getEquipmentSlot('Mainhand').setItem(afterItem)
}
system.runInterval(
    () => {
        fly();
    },
);
function extract(inputText) {
    const data = [
        ["minecraft:white_dye", "1", "1", "1"],
        ["minecraft:light_gray_dye", "0.827", "0.827", "0.827"],
        ["minecraft:gray_dye", "0.333", "0.333", "0.333"],
        ["minecraft:brown_dye", "0.4039", "0.1922", "0.0392"],
        ["minecraft:red_dye", "0.9843", "0.2118", "0.1451"],
        ["minecraft:orange_dye", "1", "0.647", "0"],
        ["minecraft:yellow_dye", "1", "1", "0"],
        ["minecraft:lime_dye", "0.502", "1", "0"],
        ["minecraft:green_dye", "0.3686", "0.6", "0.0353"],
        ["minecraft:cyan_dye", "0.1765", "0.4863", "0.6157"],
        ["minecraft:light_blue_dye", "0.3843", "0.6", "1"],
        ["minecraft:blue_dye", "0.1019", "0.3216", "0.9333"],
        ["minecraft:purple_dye", "0.502", "0", "0.502"],
        ["minecraft:magenta_dye", "1", "0", "1"],
        ["minecraft:pink_dye", "1", "0.753", "0.796"]
    ];
    const result = {};
    for (let i = 0; i < data.length; i++) {
      if (data[i][0] === inputText) {
        result.red = parseFloat(data[i][1]);
        result.green = parseFloat(data[i][2]);
        result.blue = parseFloat(data[i][3]);
        return result;
      }
    }
    return { red: 0, green: 0, blue: 0 };
};
function growCrop(target) {
    const cropList = ["minecraft:wheat", "minecraft:carrots", "minecraft:potatoes", "minecraft:beetroot", "minecraft:sweet_berry_bush", "minecraft:melon_stem", "minecraft:pumpkin_stem"];
    for (let c = 0; c < cropList.length; c++) { 
        for (let i = 0; i < 7; i++) {
            try {
                target.dimension.fillBlocks({x:target.location.x-2, y:target.location.y-50, z:target.location.z+2}, {x:target.location.x+2, y:target.location.y, z:target.location.z-2}, BlockPermutation.resolve(cropList[c], { growth: 7 }), {matchingBlock: BlockPermutation.resolve( cropList[c], { growth: i } )});
            } catch (error) {console.warn(error)};
        }
    }
}
world.afterEvents.itemUse.subscribe(eventData => {
    let item = eventData.itemStack;
    let playerUseItem = eventData.source;
    let point = {x: playerUseItem.location.x, y: playerUseItem.location.y - 2, z: playerUseItem.location.z};
    let itemToggle = ["minecraft:filled_map", "minecraft:clock", "minecraft:compass", "minecraft:recovery_compass", "minecraft:spyglass"];
    if (itemToggle.includes(item.typeId)) {playerUseItem.cameraToggle = !playerUseItem.cameraToggle;}  
    if (!playerUseItem.groundCheck && riding?.getProperty("server:dispenser_upgrade")) {
        let option = { type: "server:plane", maxDistance: 10, location: point };
        if (item.typeId.endsWith("_dye") && !playerUseItem.hasTag("smoke")) {
            molang.setColorRGB("variable.color", extract(item.typeId));
            playerUseItem.addTag("smoke");
            usedReduce(playerUseItem);
            system.runTimeout(
                () => {
                    playerUseItem.removeTag("smoke");
                }, 30,
            )
        } else if (item.typeId === "minecraft:glowstone_dust" && !playerUseItem.hasTag("dust")) {
            playerUseItem.addTag("dust");
            usedReduce(playerUseItem);
            system.runTimeout(
                () => {
                    playerUseItem.removeTag("dust");
                }, 30,
            )
        }
        switch (item.typeId) {
            case "minecraft:bone_meal":
                usedReduce(playerUseItem);
                growCrop(playerUseItem);
                playerUseItem.dimension.spawnParticle("server:bone", point)
            break;
            case "minecraft:tnt":
                usedReduce(playerUseItem);
                playerUseItem.dimension.spawnEntity("minecraft:tnt", point).applyImpulse({x:0, y:-2, z:0});
            break;
            case "minecraft:fire_charge":
                usedReduce(playerUseItem);
                playerUseItem.dimension.spawnEntity("minecraft:fireball", point).applyImpulse({x:0, y:-2, z:0});
            break;
            case "minecraft:tnt_minecart":
                usedReduce(playerUseItem);
                playerUseItem.dimension.spawnEntity("minecraft:tnt_minecart", point).applyImpulse({x:0, y:-2, z:0});
            break;
            case "minecraft:redstone":
                playerUseItem.dimension.spawnParticle("server:fuel", point);
                usedReduce(playerUseItem);
                playerUseItem.dimension.getEntities(option)[0]?.applyImpulse({x:0, y:0.75, z:0});
            break;
            case "minecraft:firework_rocket":
                usedReduce(playerUseItem);
                playerUseItem.dimension.getEntities(option)[0]?.applyImpulse({x:playerUseItem.getViewDirection().x, y:Math.min(Math.max(playerUseItem.getViewDirection().y, -1.4), 1.4), z:playerUseItem.getViewDirection().z});
                playerUseItem.addTag("sparkling");
                system.runTimeout(
                    () => {
                        playerUseItem.removeTag("sparkling");
                    }, 10,
                )
            break;
        };
    };
});

world.afterEvents.playerInteractWithEntity.subscribe(eventData => {
    let target = eventData.target;
    let item = eventData.itemStack;
    let playerUseItem = eventData.player;
    if (target.typeId === "server:plane" && item) {
        let currentFuelValue = target.getProperty("server:fuel");
        switch (item.typeId) {
            case "minecraft:dispenser":
                if (target.getProperty("server:dispenser_upgrade")) {return;}
                target.setProperty("server:dispenser_upgrade", true);
                target.playAnimation('animation.plane.shake');
                usedReduce(playerUseItem);
            break;
            case "minecraft:daylight_detector":
                if (target.getProperty("server:sensor_upgrade")) {return;}
                target.setProperty("server:sensor_upgrade", true);
                target.playAnimation('animation.plane.shake');
                usedReduce(playerUseItem);
            break;
            case "minecraft:chest":
                if (target.getProperty("server:chest_upgrade")) {return;}
                target.triggerEvent('server:add_chest');
                target.playAnimation('animation.plane.shake');
                usedReduce(playerUseItem);
            break;
            case "minecraft:redstone":
                if (currentFuelValue >= 180) {return;}
                target.playAnimation('animation.plane.shake');
                target.setProperty("server:fuel", Math.min(currentFuelValue += 20, 180));
                target.dimension.spawnParticle("server:fuel", target.location);
                usedReduce(playerUseItem);
            break;
            case "minecraft:redstone_block":
                if (currentFuelValue >= 180) {return;}
                target.playAnimation('animation.plane.shake');
                target.setProperty("server:fuel", 180);
                target.dimension.spawnParticle("server:fuel", target.location);
                usedReduce(playerUseItem);
            break;
        }
    }
});